@@ -33,6 +33,7 @@ For more information on usage visit https://github.com/Damgaard/PyImgur
 from base64 import b64encode
 import os.path
 import re
+import sys
 
 import requests
 
@@ -63,7 +64,6 @@ class Basic_object(object):
     def __getattr__(self, attribute):
         if not self._has_fetched:
             self.refresh()
-            self._has_fetched = True
             return getattr(self, attribute)
         raise AttributeError("%s instance has no attribute '%s'" %
                              (type(self).__name__, attribute))
@@ -215,6 +215,7 @@ class Basic_object(object):
         """
         resp = self._imgur._send_request(self._INFO_URL)
         self._populate(resp)
+        self._has_fetched = True
         # NOTE: What if the object has been deleted in the meantime? That might
         # give a pretty cryptic error.
 
@@ -535,7 +536,9 @@ class Image(Basic_object):
             will be used.
         :param name: The name the image will be stored as (not including file
             extension). If name is None, then the title of the image will be
-            used. If the image doesn't have a title, it's id will be used.
+            used. If the image doesn't have a title, it's id will be used. Note
+            that if the name given by name or title is an invalid filename,
+            then the hash will be used as the name instead.
         :param overwrite: If True overwrite already existing file with the same
             name as what we want to save the file as.
         :param size: Instead of downloading the image in it's original size, we
@@ -545,6 +548,14 @@ class Image(Basic_object):
 
         :returns: Name of the new file.
         """
+        def save_as(filename):
+            local_path = os.path.join(path, filename)
+            if os.path.exists(local_path) and not overwrite:
+                raise Exception("Trying to save as %s, but file already exists." %
+                                local_path)
+            with open(local_path, 'wb') as out_file:
+                out_file.write(resp.content)
+            return local_path
         valid_sizes = {'small_square': 's', 'big_square': 'b',
                        'small_thumbnail': 't', 'medium_thumbnail': 'm',
                        'large_thumbnail': 'l', 'huge_thumbnail': 'h'}
@@ -556,14 +567,13 @@ class Image(Basic_object):
         suffix = valid_sizes.get(size, '')
         base, sep, ext = self.link.rpartition('.')
         resp = requests.get(base + suffix + sep + ext)
-        filename = (name or self.title or self.id) + suffix + sep + ext
-        local_path = os.path.join(path, filename)
-        if os.path.exists(local_path) and not overwrite:
-            raise Exception("Trying to save as %s, but file already exists." %
-                            local_path)
-        with open(local_path, 'wb') as out_file:
-            out_file.write(resp.content)
-        return local_path
+        if name or self.title:
+            try:
+                return save_as((name or self.title) + suffix + sep + ext)
+            except IOError:
+                pass
+            # Invalid filename
+        return save_as(self.id + suffix + sep + ext)
 
     def favorite(self):
         """
@@ -765,10 +775,6 @@ class Imgur:
         pass
     '''
 
-#    def get_at_url(self, url):
-#        """Return whatever is at the imgur url as an object."""
-#        pass
-
     def exchange_code(self, code):
         """Exchange one-use code for an access_token and request_token."""
         params = {'client_id': self.client_id,
@@ -798,6 +804,89 @@ class Imgur:
         json = self._send_request("https://api.imgur.com/3/album/%s" % id)
         return Album(json, self)
 
+    def get_at_url(self, url):
+        """
+        Return a object representing the content at url.
+
+        Returns None if no object could be matched with the id.
+
+        Works for Album, Comment, Gallery_album, Gallery_image, Image and User.
+
+        :param url: The url where the content is located at
+        """
+        class NullDevice():
+            def write(self, s):
+                pass
+
+        def get_gallery_item(id):
+            """
+            Special helper method to get gallery items.
+
+            The problem is that it's impossible to distinguish albums and
+            images from each other based on the url. And there isn't a common
+            url endpoints that return either a Gallery_album or a Gallery_image
+            depending on what the id represents. So the only option is to
+            assume it's a Gallery_image and if we get an exception then try
+            Gallery_album.  Gallery_image is attempted first because there is
+            the most of them.
+            """
+            try:
+                # HACK: Problem is that send_request prints the error message
+                # from Imgur when it encounters an error. This is nice because
+                # this error message is more descriptive than just the status
+                # code that Requests give. But since we first assume the id
+                # belong to an image, it means we will get an error whenever
+                # the id belongs to an album. The following code temporarily
+                # disables stdout to avoid give a cryptic and incorrect error.
+
+                # Code for disabling stdout is from
+                # http://coreygoldberg.blogspot.dk/2009/05/
+                # python-redirect-or-turn-off-stdout-and.html
+                original_stdout = sys.stdout  # keep a reference to STDOUT
+                sys.stdout = NullDevice()  # redirect the real STDOUT
+                return self.get_gallery_image(id)
+            # TODO: Add better error codes so I don't have to do a catch-all
+            except Exception:
+                return self.get_gallery_album(id)
+            finally:
+                sys.stdout = original_stdout  # turn STDOUT back on
+        base = "(https?://)(i\\.)?imgur.com/"
+        objects = {'album': {'regex': "a/(?P<id>[\w.]*?)$",
+                             'method': self.get_album},
+                   'comment': {'regex': "gallery/\w*/comment/(?P<id>[\w.]*?)$",
+                               'method': self.get_comment},
+                   'gallery': {'regex': "(gallery|r/\w*?)/(?P<id>[\w.]*?)$",
+                               'method': get_gallery_item},
+                   # Valid image extensions: http://imgur.com/faq#types
+                   # All are between 3 and 4 chars long.
+                   'image': {'regex': "(?P<id>[\w.]*?)(\\.\w{3,4})?$",
+                             'method': self.get_image},
+                   'user': {'regex': "user/(?P<id>[\w.]*?)$",
+                            'method': self.get_user}
+                   }
+        for obj_type, values in objects.items():
+            regex_result = re.match(base + values['regex'], url)
+            if regex_result is not None:
+                obj_id = regex_result.group('id')
+                initial_object = values['method'](obj_id)
+                if obj_type == 'image':
+                    try:
+                        # A better version might be to ping the url where the
+                        # gallery_image should be with a requests.head call. If
+                        # we get a 200 returned, then that means it exists and
+                        # this becomes less hacky.
+                        original_stdout = sys.stdout
+                        sys.stdout = NullDevice()
+                        if getattr(initial_object, 'section', None):
+                            sub = initial_object.section
+                            return self.get_subreddit_image(sub, obj_id)
+                        return self.get_gallery_image(obj_id)
+                    except Exception:
+                        pass
+                    finally:
+                        sys.stdout = original_stdout
+                return initial_object
+
     def get_comment(self, id):
         """Return information about this comment."""
         url = "https://api.imgur.com/3/comment/%s" % id
@@ -870,6 +959,39 @@ class Imgur:
                                   id)
         return Notification(resp, self)
 
+    def get_memes_gallery(self, sort='viral', window='week', limit=None):
+        """
+        Return a list of gallery albums/images submitted to the memes gallery
+
+        The url for the memes gallery is: http://imgur.com/g/memes
+
+        :param sort: viral | time | top - defaults to viral
+        :param window: Change the date range of the request if the section is
+            "top", day | week | month | year | all, defaults to week.
+        :param limit: The number of items to return.
+        """
+        url = ("https://api.imgur.com/3/gallery/g/memes/%s/%s/%s" %
+               (sort, window, '%d'))
+        resp = self._send_request(url, limit=limit)
+        return [_get_album_or_image(thing, self) for thing in resp]
+
+    '''
+    This method does not seem to return any more data than get_gallery_image.
+    So I'm not sure whether it needs to be included. Speaking for is that
+    people may expect it to be here, speaking against is that the functionality
+    already exists and duplication will reduce usability of the API.
+    07-08-2013
+    def get_memes_image(self, id):
+        """
+        Return the Gallery_image with the id submitted to the memes gallery
+
+        :param id: The id of the image we want.
+        """
+        url = "https://api.imgur.com/3/gallery/g/memes/" % id
+        resp = self._send_request(url)
+        return Gallery_image(resp, self)
+    '''
+
     def get_subreddit_gallery(self, subreddit, sort='time', window='top',
                               limit=None):
         """
@@ -889,6 +1011,17 @@ class Imgur:
         resp = self._send_request(url, limit=limit)
         return [_get_album_or_image(thing, self) for thing in resp]
 
+    def get_subreddit_image(self, subreddit, id):
+        """
+        Return the Gallery_image with the id submitted to subreddit gallery
+
+        :param subreddit: The subreddit the image has been submitted to.
+        :param id: The id of the image we want.
+        """
+        url = "https://api.imgur.com/3/gallery/r/%s/%s" % (subreddit, id)
+        resp = self._send_request(url)
+        return Gallery_image(resp, self)
+
     def get_user(self, username):
         """
         Return a User object for this username.
@@ -931,11 +1064,13 @@ class Imgur:
         resp = self._send_request(url)
         return [_get_album_or_image(thing, self) for thing in resp]
 
-    def upload_image(self, path, title=None, description=None, album=None):
+    def upload_image(self, path=None, url=None, title=None, description=None,
+                     album=None):
         """
-        Upload the image at path.
+        Upload the image at either path or url.
 
         :param path: The path to the image you want to upload.
+        :param url: The url to the image you want to upload.
         :param title: The title the image will have when uploaded.
         :param description: The description the image will have when uploaded.
         :param album: The album the image will be added to when uploaded. Can
@@ -946,16 +1081,31 @@ class Imgur:
 
         :returns: An Image object representing the uploaded image.
         """
-        with open(path, 'rb') as image_file:
-            binary_data = image_file.read()
-            image = b64encode(binary_data)
+        if bool(path) == bool(url):
+            raise LookupError("Either path or url must be given.")
+        if path:
+            with open(path, 'rb') as image_file:
+                binary_data = image_file.read()
+                image = b64encode(binary_data)
+        else:
+            image = url
 
         payload = {'album_id': album, 'image': image,
                    'title': title, 'description': description}
 
         resp = self._send_request("https://api.imgur.com/3/image",
                                   params=payload, method='POST')
-        return Image(resp, self, False)
+        # TEMPORARY HACK:
+        # On 5-08-2013 I noticed Imgur now returned enough information from
+        # this call to fully populate the Image object. However those variables
+        # that matched arguments were always None, even if they had been given.
+        # See https://groups.google.com/forum/#!topic/imgur/F3uVb55TMGo
+        resp['title'] = title
+        resp['description'] = description
+        if album is not None:
+            resp['album'] = (Album({'id': album}, self, False) if not
+                             isinstance(album, Album) else album)
+        return Image(resp, self)
 
 
 class Message(Basic_object):
